/* Drawables
 *
 */

#include <gtkmm.h>

#include <papyrus.h>
#include <papyrusmm/viewport.h>

class Example_drawables : public Gtk::Window
{
  public:
    Example_drawables();
    virtual ~Example_drawables();

  protected:
  //Member widgets:
    PapyrusGtk::Viewport m_Viewport;
    Papyrus::Drawable::pointer m_Drawable;

    void redraw_canvas();

    Gtk::CheckButton m_fill_button, m_outline_button;
    Gtk::VBox m_vbox;
    Gtk::ComboBoxText m_combobox;
};

//Called by DemoWindow;
Gtk::Window* do_drawables()
{
  return new Example_drawables();
}

Example_drawables::Example_drawables():
    m_fill_button("Fill"),
    m_outline_button("Outline")
{
  set_title("drawables Example");
  set_border_width(10);

  m_Viewport.canvas()->set_size( 500, 400 );

  m_vbox.pack_start( m_Viewport );
  m_vbox.pack_start( *Gtk::manage( new Gtk::HSeparator() ) );
  m_vbox.pack_start( m_combobox );
  m_vbox.pack_start( m_fill_button );
  m_vbox.pack_start( m_outline_button );

  for (unsigned i=Papyrus::DRAWABLE_START; i < Papyrus::DRAWABLE_END; i++)
    m_combobox.append_text( Papyrus::DrawableStrings[i] );

  m_combobox.signal_changed().connect( sigc::mem_fun(*this, &Example_drawables::redraw_canvas));
  m_fill_button.signal_toggled().connect( sigc::mem_fun(*this, &Example_drawables::redraw_canvas));
  m_outline_button.signal_toggled().connect( sigc::mem_fun(*this, &Example_drawables::redraw_canvas));

  m_fill_button.set_active(true);
  m_outline_button.set_active(true);
  m_combobox.set_active(Papyrus::KOCHSNOWFLAKE);

  this->add( m_vbox );

  show_all();
}

Example_drawables::~Example_drawables()
{
}

void Example_drawables::redraw_canvas( )
{
  int which;
  Papyrus::Polyline::pointer line;

  which = m_combobox.get_active_row_number();

  if ( which < 0 )
    return;

  m_Viewport.canvas()->clear();
  m_Viewport.canvas()->set_background( Papyrus::RGBA("white") );

  line = Papyrus::Polyline::create();
  line->add_vertex(0, -500);
  line->add_vertex(0, 500);
  line->set_outline( Cairo::SolidPattern::create_rgb(0.0, 0.0, 1.0) );
  line->line_style().set_width(1);
  m_Viewport.canvas()->add( line );

  line = Papyrus::Polyline::create();
  line->add_vertex(-500, 0);
  line->add_vertex(500, 0);
  line->set_outline( Cairo::SolidPattern::create_rgb(0.0, 0.0, 1.0) );
  line->line_style().set_width(1);
  m_Viewport.canvas()->add( line );

  m_Drawable = Papyrus::example_factory( which,
                                         m_fill_button.get_active(),
                                         m_outline_button.get_active() );

  m_Viewport.canvas()->add( m_Drawable );
}
