/* Selector
 *
 */

#include "gtkmm.h"

#include <papyrus.h>
#include <papyrusmm/viewport.h>

class Example_Selector : public Gtk::Window
{
  public:
    Example_Selector();
    virtual ~Example_Selector();

  protected:
  //Member widgets:
    PapyrusGtk::Viewport m_Viewport;
    Papyrus::Selector::pointer m_Selector;
    Papyrus::Boxed::pointer m_boxed;

};

//Called by DemoWindow;
Gtk::Window* do_Selector()
{
  return new Example_Selector();
}

Example_Selector::Example_Selector()
{
  set_title("Selector Example");
  set_border_width(10);

  m_boxed = Papyrus::Boxed::create( );
  m_boxed->get_box()->set_fill( Cairo::SolidPattern::create_rgba(1.0, 0.85, 1.0, 0.9) );
  m_boxed->get_box()->set_outline( Cairo::SolidPattern::create_rgba(1.0, 0.0, 1.0, 0.9) );

  m_Selector = Papyrus::Selector::create(m_Viewport.canvas(), m_boxed);
  m_Viewport.add_controller( m_Selector );

  m_Viewport.canvas()->set_size( 300, 200 );

  // Create some shapes to add to the group
  Papyrus::Rectangle::pointer rectangle = Papyrus::example_rectangle( );
  Papyrus::Circle::pointer circle = Papyrus::example_circle( );
  Papyrus::Arc::pointer arc = Papyrus::example_arc( );

  // Add the shapes to the group
  m_Viewport.canvas()->add( rectangle );
  m_Viewport.canvas()->add( circle );
  m_Viewport.canvas()->add( arc );
  m_Viewport.canvas()->add( m_boxed );

  // Translate the shapes so they don't necessarily overlap
  rectangle->set_xywh(0, -60, 50, 30);
  circle->set_translate(40, 20);
  circle->set_radius(25);
  arc->set_translate(0, 20);

  this->add( m_Viewport );

  show_all();
}

Example_Selector::~Example_Selector()
{
}
