/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "beziergon.h"

namespace Papyrus {

Beziergon::Beziergon(BezierVertices vertices):
    Bezierline(vertices) {
  set_use_outline( false );
    }

    Beziergon::pointer Beziergon::create(BezierVertices vertices) {
        return Beziergon::pointer( new Beziergon(vertices) );
    }


    Beziergon::~Beziergon() {}

    void Beziergon::set_line_style(const LineStyle& line_style) {
        set_outline( line_style, true );
    }

    void Beziergon::draw_shape( Cairo::RefPtr<Cairo::Context> cairo) {

        if (m_vertices.size() < 3)
            return;
        cairo->move_to( m_vertices[0].x, m_vertices[0].y);
        BezierVertices::iterator prev=m_vertices.begin(), current=m_vertices.begin()+1;
        while ( current != m_vertices.end() )
        {
          if ( current->relative )
            cairo->rel_curve_to( prev->c2x, prev->c2y, current->c1x, current->c1y, current->x, current->y );
          else
            cairo->curve_to( prev->c2x, prev->c2y, current->c1x, current->c1y, current->x, current->y );
          prev = current++;
        }
        current = m_vertices.begin();
        cairo->curve_to( prev->c2x, prev->c2y, current->c1x, current->c1y, current->x, current->y );
        cairo->close_path();
    }

}
