/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "boxed.h"
#include "rgba.h"

#include <limits.h>

namespace Papyrus
{

  Boxed::Boxed():
      Group()
  {
    m_box = Rectangle::create();
    m_box->set_outline( LineStyle( RGBA( 1.0, 0.0, 1.0, 0.8 ), 3 ) );
    m_box->hide();
    m_box->set_exclude_from_extents( true );
    this->add( m_box, INT_MIN, false );
    m_signal_child_added.connect( sigc::mem_fun( *this, &Boxed::on_child_added_proxy ) );
    m_signal_child_removed.connect( sigc::mem_fun( *this, &Boxed::on_child_removed_proxy ) );
  }

  Boxed::pointer Boxed::create()
  {
    return Boxed::pointer( new Boxed( ) );
  }

  Boxed::~Boxed()
  {}

  bool Boxed::add( Drawable::pointer item, int layer, bool drawing_only ) {
    return Group::add(item, layer, drawing_only);
  }

  Rectangle::pointer Boxed::get_box( )
  {
    return m_box;
  }

  void Boxed::rebuild_extents()
  {
    Region extents;
    double x, y, w, h, offset = 0.0, offset_2;
    this->Group::rebuild_extents();
    extents = this->extents();
    extents.xywh( x, y, w, h );
    if ( m_box->use_outline() )
      offset = m_box->outline().width();
    offset_2 = 2.0 * offset;
    m_changed_connections[m_box].block();
    m_box->set_xywh( x - offset, y - offset, w + offset_2, h + offset_2 );
    m_changed_connections[m_box].unblock();
    extents.include(m_box->global_extents());
    this->set_extents( extents );
  }

  void Boxed::on_child_added( Drawable::pointer child )
  {
    if ( this->size() > 1 )
      m_box->show();
  }

  void Boxed::on_child_removed( Drawable::pointer child )
  {
    if ( this->size() == 1 )
      m_box->hide();
  }

  void Boxed::on_child_added_proxy( Drawable::pointer child )
  {
    this->on_child_added(child);
  }

  void Boxed::on_child_removed_proxy( Drawable::pointer child )
  {
    this->on_child_removed(child);
  }

}
