/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSKOCHSNOWFLAKE_H
#define PAPYRUSKOCHSNOWFLAKE_H

#include <papyrus/polyline.h>

namespace Papyrus
{

  /**
  	@author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
  */
  class KochSnowflake : public Polyline
  {
    public:
    typedef PapyrusSmartPointer<KochSnowflake> pointer;

    KochSnowflake(unsigned level=0, double size=0.0);

    static KochSnowflake::pointer create(unsigned level=0, double size=0.0);

    ~KochSnowflake();

    unsigned level();

    void set_level(unsigned level);

    double size();

    void set_size(double size);

    PAPYRUS_CLASS_NAME("Koch Snowflake");

    PAPYRUS_CLONE_METHOD( KochSnowflake );

  protected:
    unsigned m_level;
    double m_size;
    double m_lsize;
    double m_angle;

    void create_vertices();

    void kochsnowflake(unsigned level);

  };

}

#endif
