/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "rotator.h"

namespace Papyrus
{

  Rotator::Rotator( Group::pointer selection_group,
                    SelectionMode mode,
                    int selection_depth,
                    double radian_interval )
      : Selector( selection_group, mode, selection_depth ),
        m_radian_interval( radian_interval )
  {
    m_original_position[ 0 ] = 0.0;
    m_original_position[ 1 ] = 0.0;
    m_centroid[ 0 ] = 0.0;
    m_centroid[ 1 ] = 0.0;
  }

  Rotator::Rotator( Group::pointer select_from_group,
                          Group::pointer selection_group,
                          SelectionMode mode,
                          int selection_depth,
                          double radian_interval )
      : Selector( select_from_group, selection_group, mode, selection_depth ),
        m_radian_interval( radian_interval )
  {
    m_original_position[ 0 ] = 0.0;
    m_original_position[ 1 ] = 0.0;
    m_centroid[ 0 ] = 0.0;
    m_centroid[ 1 ] = 0.0;
  }

  Rotator::pointer Rotator::create( Group::pointer selection_group,
                                    SelectionMode mode,
                                    int selection_depth,
                                    double radian_interval )
  {
    return Rotator::pointer( new Rotator( selection_group, mode, selection_depth, radian_interval ) );
  }


  Rotator::pointer Rotator::create( Group::pointer select_from_group,
                                    Group::pointer selection_group,
                                    SelectionMode mode,
                                    int selection_depth,
                                    double radian_interval )
  {
    return Rotator::pointer( new Rotator( select_from_group, selection_group, mode, selection_depth, radian_interval ) );
  }


  Rotator::~Rotator()
  {}

//   double Rotator::radian_interval()
//   {
//     return m_radian_interval;
//   }
//
//   void Rotator::set_radian_interval( double radian_interval )
//   {
//     m_radian_interval = radian_interval;
//   }

  bool Rotator::on_button_press( const Event::ButtonPress& event )
  {
    Region ext;
    Selector::on_button_press( event );
    m_original_position[ 0 ] = event.x;
    m_original_position[ 1 ] = event.y;
    m_last_position[ 0 ] = event.x;
    m_last_position[ 1 ] = event.y;
    m_centroid[0] = 0.0;
    m_centroid[1] = 0.0;
    if ( m_selection.size() > 0 ) {
      ext = this->extents();
      m_centroid[0] = ext.left() + ext.width() / 2.0;
      m_centroid[1] = ext.top() + ext.height() / 2.0;
    }
    m_last_angle = atan2( event.y - m_centroid[1], event.x - m_centroid[0] );
    return true;
  }

  bool Rotator::on_motion( const Event::Motion & event )
  {
    Selection::iterator i;
    double tx = 0.0, ty = 0.0;
    double this_angle;

    if ( m_selection.size() == 0 )
      return false;

    this_angle = atan2( event.y - m_centroid[1], event.x - m_centroid[0] );

//     if ( m_transforms & TRANSLATE_X ) {
//       tx = event.x - m_last_position[ 0 ];
//       if ( m_grid[ 0 ] > 0.0 )
//         tx = round( tx / m_grid[ 0 ] ) * m_grid[ 0 ];
//     }
//
//     if ( m_transforms & TRANSLATE_Y ) {
//       ty = event.y - m_last_position[ 1 ];
//       if ( m_grid[ 1 ] > 0.0 )
//         ty = round( ty / m_grid[ 1 ] ) * m_grid[ 1 ];
//     }

    for ( i = m_selection.begin(); i != m_selection.end(); i++ ) {
      ( *i )->rotate( this_angle - m_last_angle );
//       if ( tx != 0.0 && ty != 0.0 ) {
//         ( *i ) ->translate( tx, ty );
//         m_last_position[ 0 ] = event.x;
//         m_last_position[ 1 ] = event.y;
//       } else if ( tx != 0.0 ) {
//         ( *i ) ->translate_x( tx );
//         m_last_position[ 0 ] = event.x;
//       } else if ( ty != 0.0 ) {
//         ( *i ) ->translate_y( ty );
//         m_last_position[ 1 ] = event.y;
//       }
    }

    m_last_angle = this_angle;

    return true;
  }



}
