/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSSCALER_H
#define PAPYRUSSCALER_H

#include <papyrus/selector.h>
#include <papyrus/handlebox.h>

namespace Papyrus
{

  /**
    @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
  class Scaler : public Selector
  {
    public:
      typedef PapyrusSmartPointer<Scaler> pointer;

    Scaler(Group::pointer selection_group = Group::pointer(),
               SelectionMode mode = SELECT_PRESS,
               int selection_depth = 1,
               unsigned side = 0 );

    Scaler(Group::pointer select_from_group,
               Group::pointer selection_group,
               SelectionMode mode = SELECT_PRESS,
               int selection_depth = 1,
               unsigned side = 0 );

    static Scaler::pointer create(Group::pointer selection_group = Group::pointer(),
                                      SelectionMode mode = SELECT_PRESS,
                                      int selection_depth = 1,
                                      unsigned side = 0 );

    static Scaler::pointer create(Group::pointer select_from_group,
                                      Group::pointer selection_group,
                                      SelectionMode mode = SELECT_PRESS,
                                      int selection_depth = 1,
                                      unsigned side = 0 );

    ~Scaler();

    virtual void set_selection_group( Group::pointer selection_group, bool add_drawable_only = true );

    unsigned sides();

    void set_sides( unsigned sides );

    void enable_sides( unsigned sides );

    void disable_sides( unsigned sides );

    /** Gets the current grid size */
//     void grid( double& x, double& y );

    /**
     * Sets the grid x and y sizes. If either is <= 0.0 then the
     * grid will be turned off for that dimension.
     */
//     void set_grid( double x, double y );

//     void set_grid_x( double x );

//     void set_grid_y( double y );

  protected:
    double m_original_position[2];
    unsigned m_sides;
    double m_grid[2];
    Handlebox::pointer m_handlebox;

    virtual bool on_motion(const Event::Motion& event);
    virtual bool on_button_press(const Event::ButtonPress& event);

  };

}

#endif
