/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "translator.h"

namespace Papyrus
{

  Translator::Translator( Group::pointer selection_group,
                          SelectionMode mode,
                          int selection_depth,
                          unsigned transforms )
      : Selector( selection_group, mode, selection_depth ),
        m_transforms( transforms )
  {
    m_original_position[ 0 ] = 0.0;
    m_original_position[ 1 ] = 0.0;
    m_grid[ 0 ] = 0.0;
    m_grid[ 1 ] = 0.0;
  }

  Translator::Translator( Group::pointer select_from_group,
                          Group::pointer selection_group,
                          SelectionMode mode,
                          int selection_depth,
                          unsigned transforms )
      : Selector( select_from_group, selection_group, mode, selection_depth ),
        m_transforms( transforms )
  {
    m_original_position[ 0 ] = 0.0;
    m_original_position[ 1 ] = 0.0;
    m_grid[ 0 ] = 0.0;
    m_grid[ 1 ] = 0.0;
  }

  Translator::pointer Translator::create( Group::pointer selection_group,
                                          SelectionMode mode,
                                          int selection_depth,
                                          unsigned transforms )
  {
    return Translator::pointer( new Translator( selection_group, mode, selection_depth, transforms ) );
  }


  Translator::pointer Translator::create( Group::pointer select_from_group,
                                          Group::pointer selection_group,
                                          SelectionMode mode,
                                          int selection_depth,
                                          unsigned transforms )
  {
    return Translator::pointer( new Translator( select_from_group, selection_group, mode, selection_depth, transforms ) );
  }


  Translator::~Translator()
  {}

  unsigned Translator::transforms()
  {
    return m_transforms;
  }

  void Translator::set_transforms( unsigned transforms )
  {
    m_transforms = transforms;
  }

  void Translator::enable_transforms( unsigned transforms )
  {
    m_transforms |= transforms;
  }

  void Translator::disable_transforms( unsigned transforms )
  {
    m_transforms &= ~transforms;
  }

  void Translator::grid( double & x, double & y )
  {
    x = m_grid[ 0 ];
    y = m_grid[ 1 ];
  }

  void Translator::set_grid( double x, double y )
  {
    m_grid[ 0 ] = x;
    m_grid[ 1 ] = y;
  }

  void Translator::set_grid_x( double x )
  {
    m_grid[ 0 ] = x;
  }

  void Translator::set_grid_y( double y )
  {
    m_grid[ 1 ] = y;
  }

  bool Translator::on_button_press( const Event::ButtonPress& event )
  {
    Selector::on_button_press( event );
    m_original_position[ 0 ] = event.x;
    m_original_position[ 1 ] = event.y;
    return true;
  }

  bool Translator::on_motion( const Event::Motion & event )
  {
    Selection::iterator i;
    double tx = 0.0, ty = 0.0;

    if ( m_transforms & TRANSLATE_X ) {
      tx = event.x - m_last_position[ 0 ];
      if ( m_grid[ 0 ] > 0.0 )
        tx = round( tx / m_grid[ 0 ] ) * m_grid[ 0 ];
    }

    if ( m_transforms & TRANSLATE_Y ) {
      ty = event.y - m_last_position[ 1 ];
      if ( m_grid[ 1 ] > 0.0 )
        ty = round( ty / m_grid[ 1 ] ) * m_grid[ 1 ];
    }

    for ( i = m_selection.begin(); i != m_selection.end(); i++ ) {
      if ( tx != 0.0 && ty != 0.0 ) {
        ( *i ) ->translate( tx, ty );
        m_last_position[ 0 ] = event.x;
        m_last_position[ 1 ] = event.y;
      } else if ( tx != 0.0 ) {
        ( *i ) ->translate_x( tx );
        m_last_position[ 0 ] = event.x;
      } else if ( ty != 0.0 ) {
        ( *i ) ->translate_y( ty );
        m_last_position[ 1 ] = event.y;
      }
    }

    return true;
  }



}
