/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "utility.h"

#include <cstdlib>

namespace PapyrusGtk {

void deallocate_pixbuf_memory(const guint8* data) {
  free(const_cast<guint8*>(data));
}

Glib::RefPtr<Gdk::Pixbuf> create_pixbuf(Papyrus::Renderable::pointer renderable, int width, int height) {
  unsigned char* data = (unsigned char*) calloc(width*height, 4);

  Cairo::RefPtr<Cairo::ImageSurface> imagesurface = Cairo::ImageSurface::create(data, Cairo::FORMAT_ARGB32, width, height, width*4);

  Cairo::RefPtr<Cairo::Context> context = Cairo::Context::create(imagesurface);
  renderable->render(context);

  unsigned char temp;
  for (int i = 0; i < width*height*4; i += 4) {
    temp = data[i];
    data[i] = data[i+2];
    data[i+2] = temp;
  }

  Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create_from_data(data, Gdk::COLORSPACE_RGB, true, 8, width, height, width*4, sigc::ptr_fun(deallocate_pixbuf_memory));

  return pixbuf;
}

}
